@@ -1,14 +1,14 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 1997-2012 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997-2013 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
  * and Distribution License("CDDL") (collectively, the "License").  You
  * may not use this file except in compliance with the License.  You can
  * obtain a copy of the License at
- * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
+ * http://glassfish.java.net/public/CDDL+GPL_1_1.html
  * or packager/legal/LICENSE.txt.  See the License for the specific
  * language governing permissions and limitations under the License.
  *
@@ -76,7 +76,8 @@ public class Base64Data implements CharSequence, Cloneable {
     // (note that having both is allowed) 
 
     private DataHandler dataHandler;
-    private byte[] data;
+    private byte[] data;    
+    private String hrefCid;
     
     /**
      * Length of the valid data in {@link #data}.
@@ -302,10 +303,11 @@ public class Base64Data implements CharSequence, Cloneable {
      * @throws IOException if i/o error occurs
      */
     public InputStream getInputStream() throws IOException {
-        if(dataHandler!=null)
+        if(dataHandler!=null) {
             return dataHandler.getInputStream();
-        else
+        } else {
             return new ByteArrayInputStream(data,0,dataLen);
+        }
     }
 
     /**
@@ -357,8 +359,9 @@ public class Base64Data implements CharSequence, Cloneable {
     }
 
     public String getMimeType() {
-        if(mimeType==null)
+        if (mimeType==null) {
             return "application/octet-stream";
+        }
         return mimeType;
     }
 
@@ -391,31 +394,35 @@ public class Base64Data implements CharSequence, Cloneable {
         case 0:
             return Base64Encoder.encode(data[base]>>2);
         case 1:
-            if(base+1<dataLen)
+            if (base+1<dataLen) {
                 b1 = data[base+1];
-            else
+            } else {
                 b1 = 0;
+            }
             return Base64Encoder.encode(
                         ((data[base]&0x3)<<4) |
                         ((b1>>4)&0xF));
         case 2:
-            if(base+1<dataLen) {
+            if (base+1<dataLen) {
                 b1 = data[base+1];
-                if(base+2<dataLen)
+                if (base+2<dataLen) {
                     b2 = data[base+2];
-                else
+                } else {
                     b2 = 0;
+                }
 
                 return Base64Encoder.encode(
                             ((b1&0xF)<<2)|
                             ((b2>>6)&0x3));
-            } else
+            } else {
                 return '=';
+            }
         case 3:
-            if(base+2<dataLen)
+            if(base+2<dataLen) {
                 return Base64Encoder.encode(data[base+2]&0x3F);
-            else
+            } else {
                 return '=';
+            }
         }
 
         throw new IllegalStateException();
@@ -429,8 +436,9 @@ public class Base64Data implements CharSequence, Cloneable {
     public CharSequence subSequence(int start, int end) {
         StringBuilder buf = new StringBuilder();
         get();  // fill in the buffer if we haven't done so
-        for( int i=start; i<end; i++ )
+        for (int i=start; i<end; i++ ) {
             buf.append(charAt(i));
+        }
         return buf;
     }
 
@@ -488,7 +496,25 @@ public class Base64Data implements CharSequence, Cloneable {
     
     @Override
     public Base64Data clone() {
-        return new Base64Data(this);
+        try {
+            Base64Data clone = (Base64Data) super.clone();
+            clone.get();
+            if (clone.dataCloneByRef) {
+                this.data = clone.data;
+            } else {
+                this.data = new byte[clone.dataLen];
+                System.arraycopy(clone.data, 0, this.data, 0, clone.dataLen);
+            }
+
+            this.dataCloneByRef = true;
+            this.dataLen = clone.dataLen;
+            this.dataHandler = null;
+            this.mimeType = clone.mimeType;
+            return clone;
+        } catch (CloneNotSupportedException ex) {
+            Logger.getLogger(Base64Data.class.getName()).log(Level.SEVERE, null, ex);
+            return null;
+        }
     }
 
     static String getProperty(final String propName) {
@@ -520,5 +546,17 @@ public class Base64Data implements CharSequence, Cloneable {
 //        //System.out.println("SW: " + sw.toString());
 //
 //    }
+
+    public String getHrefCid() {
+        if (hrefCid == null && dataHandler != null && dataHandler instanceof StreamingDataHandler) {
+            hrefCid = ((StreamingDataHandler)dataHandler).getHrefCid();
+        }
+        return hrefCid;
+    }
+
+    public void setHrefCid(final String cid) {
+        this.hrefCid = cid;
+        if (dataHandler != null && dataHandler instanceof StreamingDataHandler) ((StreamingDataHandler)dataHandler).setHrefCid(cid);
+    }
     
 }
